@@ -24,9 +24,9 @@
 #include <string.h>
 #include <time.h>
 
-#include <eel/eel.h>
 #include <gnome.h>
 
+#include "seahorse-gconf.h"
 #include "seahorse-gpgmex.h"
 #include "seahorse-agent.h"
 #include "seahorse-agent-secmem.h"
@@ -39,10 +39,6 @@
  * if their TTL expires. 
  */
 
-#define SETTING_CACHE       "/apps/seahorse/agent/cache_enabled"
-#define SETTING_TTL         "/apps/seahorse/agent/cache_ttl"
-#define SETTING_EXPIRE      "/apps/seahorse/agent/cache_expire"
-
 #define UNPARSEABLE_KEY     _("Unparseable Key ID")
 #define UNKNOWN_KEY         _("Unknown/Invalid Key")
 #define TRANSIENT_ID        "TRANSIENT"
@@ -97,14 +93,14 @@ seahorse_agent_cache_check (gpointer unused)
         return FALSE;
 
     if (g_hash_table_size (g_cache) > 0) {
-        if (!eel_gconf_get_boolean (SETTING_CACHE)) {
+        if (!seahorse_gconf_get_boolean (SETTING_CACHE)) {
             /* No caching */
             ttl = 0;
         }
 
-        else if (eel_gconf_get_boolean (SETTING_EXPIRE)) {
+        else if (seahorse_gconf_get_boolean (SETTING_EXPIRE)) {
             /* How long to cache. gconf has in minutes, we want seconds */
-            ttl = eel_gconf_get_integer (SETTING_TTL) * 60;
+            ttl = seahorse_gconf_get_integer (SETTING_TTL) * 60;
         }
 
         /* negative means cache indefinitely */
@@ -309,10 +305,10 @@ seahorse_agent_cache_set (const gchar *id, const gchar *pass,
     if (id == NULL)
         id = TRANSIENT_ID;
 
-    g_assert (pass && pass[0]);
+    g_return_if_fail (pass != NULL);
 
     /* Whether to cache passwords or not */
-    cache = eel_gconf_get_boolean (SETTING_CACHE);
+    cache = seahorse_gconf_get_boolean (SETTING_CACHE);
 
     /* No need to even bother the cache in this case */
     if (!cache && !lock)
