@@ -123,7 +123,7 @@ window_update_keys ()
         gtk_tree_view_set_model (tree, GTK_TREE_MODEL (store));
 
         /* Make the column */
-        column = gtk_tree_view_column_new_with_attributes (_("Cached GPG Keys"),
+        column = gtk_tree_view_column_new_with_attributes (_("Cached Encryption Keys"),
                                                            gtk_cell_renderer_text_new
                                                            (), "text", KEY_COLUMN,
                                                            NULL);
@@ -143,7 +143,6 @@ static void
 window_show ()
 {
     GtkWidget *w;
-    GtkImage *img;
     
     if (g_window) {
         w = glade_xml_get_widget (g_window->xml, g_window->name);
