@@ -20,7 +20,6 @@
  */
 
 #include <string.h>
-#include <eel/eel.h>
 
 #include "seahorse-gpgmex.h"
 #include "seahorse-op.h"
@@ -28,6 +27,7 @@
 #include "seahorse-context.h"
 #include "seahorse-key-source.h"
 #include "seahorse-vfs-data.h"
+#include "seahorse-gconf.h"
 
 /* helper function for importing @data. @data will be released.
  * returns number of keys imported or -1. */
@@ -223,7 +223,7 @@ seahorse_op_export_text (GList *keys, gboolean complete, GError **err)
     
 	/* export data with armor */
 	if (export_data (keys, complete, TRUE, data, err)) {
-    	return seahorse_util_write_data_to_text (data);
+    	return seahorse_util_write_data_to_text (data, TRUE);
     } else {
         gpgme_data_release (data);
         return NULL;
@@ -250,8 +250,8 @@ encrypt_data_common (SeahorseKeySource *sksrc, GList *keys, gpgme_data_t plain,
 	if (GPG_IS_OK (*err)) {
 
         /* Add the default key if set and necessary */
-        if (eel_gconf_get_boolean (ENCRYPTSELF_KEY)) {
-            id = eel_gconf_get_string (DEFAULT_KEY);        
+        if (seahorse_gconf_get_boolean (ENCRYPTSELF_KEY)) {
+            id = seahorse_gconf_get_string (DEFAULT_KEY);        
             if (id != NULL) {
                 skey = seahorse_key_source_get_key (sksrc, id);
                 if (skey != NULL)
@@ -262,7 +262,7 @@ encrypt_data_common (SeahorseKeySource *sksrc, GList *keys, gpgme_data_t plain,
         /* Make keys into the right format */
         recips = seahorse_util_keylist_to_keys (keys);
         
-        gpgme_set_armor (sksrc->ctx, force_armor || eel_gconf_get_boolean (ARMOR_KEY));
+        gpgme_set_armor (sksrc->ctx, force_armor || seahorse_gconf_get_boolean (ARMOR_KEY));
 		*err = func (sksrc->ctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, plain, cipher);
         
         seahorse_util_free_keys (recips);
@@ -335,7 +335,7 @@ encrypt_text_common (GList *keys, const gchar *text, EncryptFunc func,
 	encrypt_data_common (sksrc, keys, plain, cipher, func, TRUE, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	
-	return seahorse_util_write_data_to_text (cipher);
+	return seahorse_util_write_data_to_text (cipher, TRUE);
 }
 
 /**
@@ -431,7 +431,7 @@ seahorse_op_sign_file (SeahorseKeyPair *signer, const gchar *path,
     
 	/* get detached signature */
     gpgme_set_textmode (sksrc->ctx, FALSE);
-    gpgme_set_armor (sksrc->ctx, eel_gconf_get_boolean (ARMOR_KEY));    
+    gpgme_set_armor (sksrc->ctx, seahorse_gconf_get_boolean (ARMOR_KEY));    
 	sign_data (sksrc, plain, sig, GPGME_SIG_MODE_DETACH, err);
 	g_return_if_fail (GPG_IS_OK (*err));
   
@@ -478,7 +478,7 @@ seahorse_op_sign_text (SeahorseKeyPair *signer, const gchar *text,
 	sign_data (sksrc, plain, sig, GPGME_SIG_MODE_CLEAR, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	
-	return seahorse_util_write_data_to_text (sig);
+	return seahorse_util_write_data_to_text (sig, TRUE);
 }
 
 /**
@@ -600,7 +600,7 @@ seahorse_op_verify_text (SeahorseKeySource *sksrc, const gchar *text,
     gpgme_data_release (sig);     
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	/* return verified text */
-	return seahorse_util_write_data_to_text (plain);
+	return seahorse_util_write_data_to_text (plain, TRUE);
 }
 
 /* helper function to decrypt and verify @cipher. @cipher will be released. */
@@ -688,5 +688,5 @@ seahorse_op_decrypt_verify_text (SeahorseKeySource *sksrc, const gchar *text,
 	decrypt_verify_data (sksrc, cipher, plain, status, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	/* return text of decrypted data */
-	return seahorse_util_write_data_to_text (plain);
+	return seahorse_util_write_data_to_text (plain, TRUE);
 }
