@@ -27,6 +27,7 @@
 #include "seahorse-gpgmex.h"
 #include "seahorse-operation.h"
 #include "seahorse-ldap-source.h"
+#include "seahorse-hkp-source.h"
 #include "seahorse-server-source.h"
 #include "seahorse-multi-source.h"
 #include "seahorse-util.h"
@@ -568,7 +569,7 @@ parse_keyserver_uri (char *uri, const char **scheme, const char **host)
 
     *scheme = strsep(&uri, ":");
     if (uri == NULL) {
-        /* Assume HKP if there is no scheme */
+        /* Assume LDAP if there is no scheme */
         assume_ldap = 1;
         uri = (char*)*scheme;
         *scheme = "ldap";
@@ -618,6 +619,15 @@ seahorse_server_source_new (SeahorseKeySource *locsrc, const gchar *server,
             ssrc = SEAHORSE_SERVER_SOURCE (seahorse_ldap_source_new (locsrc, host, pattern));
         else
 #endif /* WITH_LDAP */
+        
+#ifdef WITH_HKP
+        if (g_ascii_strcasecmp (scheme, "hkp") == 0 || 
+            g_ascii_strcasecmp (scheme, "http") == 0 ||
+            g_ascii_strcasecmp (scheme, "https") == 0)
+            ssrc = SEAHORSE_SERVER_SOURCE (seahorse_hkp_source_new (locsrc, host, pattern));
+        else
+#endif /* WITH_HKP */
+        
             g_warning ("unsupported keyserver uri scheme: %s", scheme);
     }
     
@@ -625,5 +635,42 @@ seahorse_server_source_new (SeahorseKeySource *locsrc, const gchar *server,
     return ssrc;
 }
 
-gboolean    
-seahorse_server_source_valid_uri         (const gchar *server);
+GSList*
+seahorse_server_source_get_types()
+{
+    GSList *types = NULL;
+#ifdef WITH_LDAP
+    types = g_slist_prepend(types, g_strdup("ldap"));
+#endif
+#ifdef WITH_HKP
+    types = g_slist_prepend(types, g_strdup("hkp"));
+#endif
+    return types;
+}
+
+GSList*
+seahorse_server_source_get_descriptions()
+{
+    GSList *descriptions = NULL;
+#ifdef WITH_LDAP
+    descriptions = g_slist_prepend(descriptions, g_strdup(_("LDAP Key Server")));
+#endif
+#ifdef WITH_HKP
+    descriptions = g_slist_prepend(descriptions, g_strdup(_("HTTP Key Server")));
+#endif
+    return descriptions;
+}
+
+gboolean
+seahorse_server_source_valid_uri (const gchar *uri)
+{
+#ifdef WITH_LDAP
+    if (seahorse_ldap_is_valid_uri (uri))
+        return TRUE;
+#endif
+#ifdef WITH_HKP
+    if (seahorse_hkp_is_valid_uri (uri))
+        return TRUE;
+#endif
+    return FALSE;
+}
