@@ -21,6 +21,7 @@
 
 #include <gnome.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-windows.h"
 #include "seahorse-key-widget.h"
 #include "seahorse-key-op.h"
@@ -51,9 +52,9 @@ ok_clicked (SeahorseWidget *swidget)
 	glade_xml_get_widget (swidget->xml, "expires"))))
 		options = options | SIGN_EXPIRES;
 	
-	err = seahorse_key_op_sign (swidget->sctx, skwidget->skey, skwidget->index, check, options);
+	err = seahorse_key_op_sign (skwidget->skey, skwidget->index, check, options);
 	if (!GPG_IS_OK (err)) {
-		seahorse_util_handle_error (err, _("Couldn't sign key"));
+		seahorse_util_handle_gpgme (err, _("Couldn't sign key"));
 		return FALSE;
 	}
 	else {
